@@ -58,6 +58,7 @@ from pymongo.errors import (  # type:ignore[attr-defined]
     WaitQueueTimeoutError,
 )
 from pymongo.hello import Hello, HelloCompat
+from pymongo.helpers_shared import _get_timeout_details, format_timeout_details
 from pymongo.lock import (
     _async_cond_wait,
     _async_create_condition,
@@ -79,9 +80,7 @@ from pymongo.pool_shared import (
     SSLErrors,
     _CancellationContext,
     _configured_protocol_interface,
-    _get_timeout_details,
     _raise_connection_failure,
-    format_timeout_details,
 )
 from pymongo.read_preferences import ReadPreference
 from pymongo.server_api import _add_to_command
@@ -105,21 +104,6 @@ if TYPE_CHECKING:
     from pymongo.typings import _Address, _CollationIn
     from pymongo.write_concern import WriteConcern
 
-try:
-    from fcntl import F_GETFD, F_SETFD, FD_CLOEXEC, fcntl
-
-    def _set_non_inheritable_non_atomic(fd: int) -> None:
-        """Set the close-on-exec flag on the given file descriptor."""
-        flags = fcntl(fd, F_GETFD)
-        fcntl(fd, F_SETFD, flags | FD_CLOEXEC)
-
-except ImportError:
-    # Windows, various platforms we don't claim to support
-    # (Jython, IronPython, ..), systems that don't provide
-    # everything we need from fcntl, etc.
-    def _set_non_inheritable_non_atomic(fd: int) -> None:  # noqa: ARG001
-        """Dummy function for platforms that don't provide fcntl."""
-
 
 _IS_SYNC = False
 
@@ -131,6 +115,7 @@ class AsyncConnection:
     :param pool: a Pool instance
     :param address: the server's (host, port)
     :param id: the id of this socket in it's pool
+    :param is_sdam: SDAM connections do not call hello on creation
     """
 
     def __init__(
@@ -139,11 +124,13 @@ class AsyncConnection:
         pool: Pool,
         address: tuple[str, int],
         id: int,
+        is_sdam: bool,
     ):
         self.pool_ref = weakref.ref(pool)
         self.conn = conn
         self.address = address
         self.id = id
+        self.is_sdam = is_sdam
         self.closed = False
         self.last_checkin_time = time.monotonic()
         self.performed_handshake = False
@@ -198,7 +185,7 @@ class AsyncConnection:
         self.conn.get_conn.settimeout(timeout)
 
     def apply_timeout(
-        self, client: AsyncMongoClient, cmd: Optional[MutableMapping[str, Any]]
+        self, client: AsyncMongoClient[Any], cmd: Optional[MutableMapping[str, Any]]
     ) -> Optional[float]:
         # CSOT: use remaining timeout when set.
         timeout = _csot.remaining()
@@ -252,7 +239,7 @@ class AsyncConnection:
         else:
             return {HelloCompat.LEGACY_CMD: 1, "helloOk": True}
 
-    async def hello(self) -> Hello:
+    async def hello(self) -> Hello[dict[str, Any]]:
         return await self._hello(None, None)
 
     async def _hello(
@@ -354,7 +341,7 @@ class AsyncConnection:
         dbname: str,
         spec: MutableMapping[str, Any],
         read_preference: _ServerMode = ReadPreference.PRIMARY,
-        codec_options: CodecOptions = DEFAULT_CODEC_OPTIONS,
+        codec_options: CodecOptions[Mapping[str, Any]] = DEFAULT_CODEC_OPTIONS,  # type: ignore[assignment]
         check: bool = True,
         allowable_errors: Optional[Sequence[Union[str, int]]] = None,
         read_concern: Optional[ReadConcern] = None,
@@ -362,7 +349,7 @@ class AsyncConnection:
         parse_write_concern_error: bool = False,
         collation: Optional[_CollationIn] = None,
         session: Optional[AsyncClientSession] = None,
-        client: Optional[AsyncMongoClient] = None,
+        client: Optional[AsyncMongoClient[Any]] = None,
         retryable_write: bool = False,
         publish_events: bool = True,
         user_fields: Optional[Mapping[str, Any]] = None,
@@ -414,7 +401,7 @@ class AsyncConnection:
                 spec,
                 self.is_mongos,
                 read_preference,
-                codec_options,
+                codec_options,  # type: ignore[arg-type]
                 session,
                 client,
                 check,
@@ -486,7 +473,7 @@ class AsyncConnection:
         await self.send_message(msg, max_doc_size)
 
     async def write_command(
-        self, request_id: int, msg: bytes, codec_options: CodecOptions
+        self, request_id: int, msg: bytes, codec_options: CodecOptions[Mapping[str, Any]]
     ) -> dict[str, Any]:
         """Send "insert" etc. command, returning response as a dict.
 
@@ -538,7 +525,7 @@ class AsyncConnection:
                 )
 
     def validate_session(
-        self, client: Optional[AsyncMongoClient], session: Optional[AsyncClientSession]
+        self, client: Optional[AsyncMongoClient[Any]], session: Optional[AsyncClientSession]
     ) -> None:
         """Validate this session before use with client.
 
@@ -595,7 +582,7 @@ class AsyncConnection:
         self,
         command: MutableMapping[str, Any],
         session: Optional[AsyncClientSession],
-        client: Optional[AsyncMongoClient],
+        client: Optional[AsyncMongoClient[Any]],
     ) -> None:
         """Add $clusterTime."""
         if client:
@@ -626,7 +613,7 @@ class AsyncConnection:
         # signals and throws KeyboardInterrupt into the current frame on the
         # main thread.
         #
-        # But in Gevent and Eventlet, the polling mechanism (epoll, kqueue,
+        # But in Gevent, the polling mechanism (epoll, kqueue,
         # ..) is called in Python code, which experiences the signal as a
         # KeyboardInterrupt from the start, rather than as an initial
         # socket.error, so we catch that, close the socket, and reraise it.
@@ -704,20 +691,18 @@ class PoolState:
     CLOSED = 3
 
 
-# Do *not* explicitly inherit from object or Jython won't call __del__
-# https://bugs.jython.org/issue1057
 class Pool:
     def __init__(
         self,
         address: _Address,
         options: PoolOptions,
-        handshake: bool = True,
+        is_sdam: bool = False,
         client_id: Optional[ObjectId] = None,
     ):
         """
         :param address: a (hostname, port) tuple
         :param options: a PoolOptions instance
-        :param handshake: whether to call hello for each new AsyncConnection
+        :param is_sdam: whether to call hello for each new AsyncConnection
         """
         if options.pause_enabled:
             self.state = PoolState.PAUSED
@@ -729,7 +714,7 @@ class Pool:
         # LIFO pool. Sockets are ordered on idle time. Sockets claimed
         # and returned to pool from the left side. Stale sockets removed
         # from the right side.
-        self.conns: collections.deque = collections.deque()
+        self.conns: collections.deque[AsyncConnection] = collections.deque()
         self.active_contexts: set[_CancellationContext] = set()
         self.lock = _async_create_lock()
         self._max_connecting_cond = _async_create_condition(self.lock)
@@ -746,14 +731,14 @@ class Pool:
         self.pid = os.getpid()
         self.address = address
         self.opts = options
-        self.handshake = handshake
+        self.is_sdam = is_sdam
         # Don't publish events or logs in Monitor pools.
         self.enabled_for_cmap = (
-            self.handshake
+            not self.is_sdam
             and self.opts._event_listeners is not None
             and self.opts._event_listeners.enabled_for_cmap
         )
-        self.enabled_for_logging = self.handshake
+        self.enabled_for_logging = not self.is_sdam
 
         # The first portion of the wait queue.
         # Enforces: maxPoolSize
@@ -836,8 +821,8 @@ class Pool:
             if service_id is None:
                 sockets, self.conns = self.conns, collections.deque()
             else:
-                discard: collections.deque = collections.deque()
-                keep: collections.deque = collections.deque()
+                discard: collections.deque = collections.deque()  # type: ignore[type-arg]
+                keep: collections.deque = collections.deque()  # type: ignore[type-arg]
                 for conn in self.conns:
                     if conn.service_id == service_id:
                         discard.append(conn)
@@ -863,7 +848,7 @@ class Pool:
         if close:
             if not _IS_SYNC:
                 await asyncio.gather(
-                    *[conn.close_conn(ConnectionClosedReason.POOL_CLOSED) for conn in sockets],
+                    *[conn.close_conn(ConnectionClosedReason.POOL_CLOSED) for conn in sockets],  # type: ignore[func-returns-value]
                     return_exceptions=True,
                 )
             else:
@@ -900,7 +885,7 @@ class Pool:
                     )
             if not _IS_SYNC:
                 await asyncio.gather(
-                    *[conn.close_conn(ConnectionClosedReason.STALE) for conn in sockets],
+                    *[conn.close_conn(ConnectionClosedReason.STALE) for conn in sockets],  # type: ignore[func-returns-value]
                     return_exceptions=True,
                 )
             else:
@@ -914,7 +899,7 @@ class Pool:
         self.is_writable = is_writable
         async with self.lock:
             for _socket in self.conns:
-                _socket.update_is_writable(self.is_writable)
+                _socket.update_is_writable(self.is_writable)  # type: ignore[arg-type]
 
     async def reset(
         self, service_id: Optional[ObjectId] = None, interrupt_connections: bool = False
@@ -953,7 +938,7 @@ class Pool:
                     close_conns.append(self.conns.pop())
             if not _IS_SYNC:
                 await asyncio.gather(
-                    *[conn.close_conn(ConnectionClosedReason.IDLE) for conn in close_conns],
+                    *[conn.close_conn(ConnectionClosedReason.IDLE) for conn in close_conns],  # type: ignore[func-returns-value]
                     return_exceptions=True,
                 )
             else:
@@ -1058,14 +1043,14 @@ class Pool:
 
             raise
 
-        conn = AsyncConnection(networking_interface, self, self.address, conn_id)  # type: ignore[arg-type]
+        conn = AsyncConnection(networking_interface, self, self.address, conn_id, self.is_sdam)  # type: ignore[arg-type]
         async with self.lock:
             self.active_contexts.add(conn.cancel_context)
             self.active_contexts.discard(tmp_context)
         if tmp_context.cancelled:
             conn.cancel_context.cancel()
         try:
-            if self.handshake:
+            if not self.is_sdam:
                 await conn.hello()
                 self.is_writable = conn.is_writable
             if handler:
@@ -1474,4 +1459,4 @@ class Pool:
         # not safe to acquire a lock in __del__.
         if _IS_SYNC:
             for conn in self.conns:
-                conn.close_conn(None)
+                conn.close_conn(None)  # type: ignore[unused-coroutine]
