@@ -512,6 +512,10 @@ class ClientSession:
         # Is this an implicitly created session?
         self._implicit = implicit
         self._transaction = _Transaction(None, client)
+        # Is this session attached to a cursor?
+        self._attached_to_cursor = False
+        # Should we leave the session alive when the cursor is closed?
+        self._leave_alive = False
 
     def end_session(self) -> None:
         """Finish this session. If a transaction has started, abort it.
@@ -534,7 +538,7 @@ class ClientSession:
 
     def _end_implicit_session(self) -> None:
         # Implicit sessions can't be part of transactions or pinned connections
-        if self._server_session is not None:
+        if not self._leave_alive and self._server_session is not None:
             self._client._return_server_session(self._server_session)
             self._server_session = None
 
